import _slicedToArray from "@babel/runtime/helpers/esm/slicedToArray";
import _objectWithoutProperties from "@babel/runtime/helpers/esm/objectWithoutProperties";
import React, { lazy, useState, useEffect, Suspense } from 'react';
import PropTypes from 'prop-types';
import cx from 'classnames';
import markerFactory from './markerFactory';
var BASE_CLASS = "sui-AtomSlider";
var CLASS_DISABLED = BASE_CLASS + "--disabled";
var Range = lazy(function () {
  return import('rc-slider/lib/Range');
});
var Slider = lazy(function () {
  return import('rc-slider/lib/Slider');
});
var Tooltip = lazy(function () {
  return import('rc-tooltip');
});
var Label = lazy(function () {
  return import('./Label');
});

var createHandler = function createHandler(refAtomSlider, handleComponent, hideTooltip) {
  return function (props) {
    var value = props.value,
        index = props.index,
        dragging = props.dragging,
        restProps = _objectWithoutProperties(props, ["value", "index", "dragging"]); // eslint-disable-line


    var Handle = handleComponent.component;

    if (hideTooltip) {
      return React.createElement(Handle, Object.assign({
        value: value
      }, restProps, {
        dragging: dragging.toString()
      }));
    }

    return React.createElement(Tooltip, {
      getTooltipContainer: function getTooltipContainer() {
        return refAtomSlider.current;
      },
      key: index,
      overlay: value,
      placement: "top",
      prefixCls: "rc-slider-tooltip",
      visible: true
    }, React.createElement(Handle, Object.assign({
      value: value
    }, restProps, {
      dragging: dragging.toString()
    })));
  };
};

var AtomSlider = function AtomSlider(_ref) {
  var _cx;

  var onChange = _ref.onChange,
      value = _ref.value,
      min = _ref.min,
      max = _ref.max,
      step = _ref.step,
      range = _ref.range,
      disabled = _ref.disabled,
      valueLabel = _ref.valueLabel,
      marks = _ref.marks,
      valueLabelFormatter = _ref.valueLabelFormatter,
      hideTooltip = _ref.hideTooltip;

  var _useState = useState(false),
      _useState2 = _slicedToArray(_useState, 2),
      ready = _useState2[0],
      setReady = _useState2[1];

  var _useState3 = useState({
    component: null
  }),
      _useState4 = _slicedToArray(_useState3, 2),
      handleComponent = _useState4[0],
      setHandle = _useState4[1];

  var _useState5 = useState(value || min),
      _useState6 = _slicedToArray(_useState5, 2),
      labelValue = _useState6[0],
      setLabelValue = _useState6[1];

  var refAtomSlider = React.createRef();
  useEffect(function () {
    // import Handle here and set it in the state as tooltip need this to be loaded
    // before trying to be shown, otherwise, the reference is wrong
    // and the tooltip is not positioned correctly
    import('rc-slider/lib/Handle').then(function (_ref2) {
      var Handle = _ref2["default"];
      setHandle({
        component: Handle
      });
      setReady(true);
    });
  }, []);

  var handleChange = function handleChange(value) {
    var e = {};
    setLabelValue(value);
    onChange(e, {
      value: value
    });
  };

  var customProps = {
    defaultValue: range ? [min, max] : value,
    handle: createHandler(refAtomSlider, handleComponent, hideTooltip),
    onChange: handleChange,
    disabled: disabled,
    marks: markerFactory({
      step: step,
      min: min,
      max: max,
      marks: marks
    }),
    max: max,
    min: min,
    step: step
  }; // Determine the type of the slider according to the range prop

  var Type = range ? Range : Slider;
  return React.createElement("div", {
    ref: refAtomSlider,
    className: cx(BASE_CLASS, (_cx = {}, _cx[CLASS_DISABLED] = disabled, _cx))
  }, ready && React.createElement(Suspense, {
    fallback: null
  }, valueLabel ? React.createElement(React.Fragment, null, React.createElement(Label, {
    value: labelValue,
    formatter: valueLabelFormatter
  }), React.createElement(Type, customProps)) : React.createElement(Type, customProps)));
};

AtomSlider.displayName = 'AtomSlider';
AtomSlider.defaultProps = {
  min: 0,
  max: 100,
  step: 1,
  onChange: function onChange() {},
  hideTooltip: false
};
export default AtomSlider;